@@ -20,6 +20,7 @@ import time
 
 from concurrent import futures
 import msgpack
+from oslo_utils import encodeutils
 import six
 import sysv_ipc
 
@@ -133,6 +134,15 @@ class IPCDriver(coordination.CoordinationDriver):
     .. _IPC: http://en.wikipedia.org/wiki/Inter-process_communication
     """
 
+    CHARACTERISTICS = (
+        coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
+    )
+    """
+    Tuple of :py:class:`~tooz.coordination.Characteristics` introspectable
+    enum member(s) that can be used to interogate how this driver works.
+    """
+
     _SEGMENT_SIZE = 1024
     _GROUP_LIST_KEY = "GROUP_LIST"
     _GROUP_PROJECT = "_TOOZ_INTERNAL"
@@ -243,7 +253,7 @@ class IPCFutureResult(coordination.CoordAsyncResult):
             return self._fut.result(timeout=timeout)
         except futures.TimeoutError as e:
             coordination.raise_with_cause(coordination.OperationTimedOut,
-                                          utils.exception_message(e),
+                                          encodeutils.exception_to_unicode(e),
                                           cause=e)
 
     def done(self):
