@@ -22,6 +22,7 @@ import logging
 import string
 
 from concurrent import futures
+from oslo_utils import encodeutils
 from oslo_utils import strutils
 import redis
 from redis import exceptions
@@ -46,11 +47,11 @@ def _translate_failures():
         yield
     except (exceptions.ConnectionError, exceptions.TimeoutError) as e:
         coordination.raise_with_cause(coordination.ToozConnectionError,
-                                      utils.exception_message(e),
+                                      encodeutils.exception_to_unicode(e),
                                       cause=e)
     except exceptions.RedisError as e:
         coordination.raise_with_cause(coordination.ToozError,
-                                      utils.exception_message(e),
+                                      encodeutils.exception_to_unicode(e),
                                       cause=e)
 
 
@@ -160,6 +161,17 @@ class RedisDriver(coordination._RunWatchersMixin,
     .. _AOF: http://redis.io/topics/persistence
     """
 
+    CHARACTERISTICS = (
+        coordination.Characteristics.DISTRIBUTED_ACROSS_THREADS,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_PROCESSES,
+        coordination.Characteristics.DISTRIBUTED_ACROSS_HOSTS,
+        coordination.Characteristics.CAUSAL,
+    )
+    """
+    Tuple of :py:class:`~tooz.coordination.Characteristics` introspectable
+    enum member(s) that can be used to interogate how this driver works.
+    """
+
     MIN_VERSION = version.LooseVersion("2.6.0")
     """
     The min redis version that this driver requires to operate with...
@@ -229,9 +241,9 @@ class RedisDriver(coordination._RunWatchersMixin,
 
     #: Client arguments that are expected to be int convertible.
     CLIENT_INT_ARGS = frozenset([
-         'db',
-         'socket_keepalive',
-         'socket_timeout',
+        'db',
+        'socket_keepalive',
+        'socket_timeout',
     ])
 
     #: Default socket timeout to use when none is provided.
@@ -314,12 +326,12 @@ return 1
         lock_timeout = options.get('lock_timeout', self.timeout)
         self.lock_timeout = int(lock_timeout)
         namespace = options.get('namespace', self.DEFAULT_NAMESPACE)
-        self._namespace = self._to_binary(namespace)
+        self._namespace = utils.to_binary(namespace, encoding=self._encoding)
         self._group_prefix = self._namespace + b"_group"
         self._beat_prefix = self._namespace + b"_beats"
         self._groups = self._namespace + b"_groups"
         self._client = None
-        self._member_id = self._to_binary(member_id)
+        self._member_id = utils.to_binary(member_id, encoding=self._encoding)
         self._acquired_locks = set()
         self._joined_groups = set()
         self._executor = utils.ProxyExecutor.build("Redis", options)
@@ -336,7 +348,7 @@ return 1
             raise TypeError("Version check expects a string/version type")
         try:
             redis_version = version.LooseVersion(
-                    self._server_info['redis_version'])
+                self._server_info['redis_version'])
         except KeyError:
             return (not_existent, None)
         else:
@@ -345,11 +357,6 @@ return 1
             else:
                 return (True, redis_version)
 
-    def _to_binary(self, text):
-        if not isinstance(text, six.binary_type):
-            text = text.encode(self._encoding)
-        return text
-
     @property
     def namespace(self):
         return self._namespace
@@ -420,7 +427,7 @@ return 1
                                              self.timeout)
         except exceptions.RedisError as e:
             coordination.raise_with_cause(coordination.ToozConnectionError,
-                                          utils.exception_message(e),
+                                          encodeutils.exception_to_unicode(e),
                                           cause=e)
         else:
             # Ensure that the server is alive and not dead, this does not
@@ -461,30 +468,30 @@ return 1
             self._started = True
 
     def _encode_beat_id(self, member_id):
-        return self.NAMESPACE_SEP.join([self._beat_prefix,
-                                        self._to_binary(member_id)])
+        member_id = utils.to_binary(member_id, encoding=self._encoding)
+        return self.NAMESPACE_SEP.join([self._beat_prefix, member_id])
 
     def _encode_member_id(self, member_id):
-        member_id = self._to_binary(member_id)
+        member_id = utils.to_binary(member_id, encoding=self._encoding)
         if member_id == self.GROUP_EXISTS:
             raise ValueError("Not allowed to use private keys as a member id")
         return member_id
 
     def _decode_member_id(self, member_id):
-        return self._to_binary(member_id)
+        return utils.to_binary(member_id, encoding=self._encoding)
 
     def _encode_group_leader(self, group_id):
-        group_id = self._to_binary(group_id)
+        group_id = utils.to_binary(group_id, encoding=self._encoding)
         return b"leader_of_" + group_id
 
     def _encode_group_id(self, group_id, apply_namespace=True):
-        group_id = self._to_binary(group_id)
+        group_id = utils.to_binary(group_id, encoding=self._encoding)
         if not apply_namespace:
             return group_id
         return self.NAMESPACE_SEP.join([self._group_prefix, group_id])
 
     def _decode_group_id(self, group_id):
-        return self._to_binary(group_id)
+        return utils.to_binary(group_id, encoding=self._encoding)
 
     def heartbeat(self):
         with _translate_failures():
@@ -784,7 +791,7 @@ class RedisFutureResult(coordination.CoordAsyncResult):
                 return self._fut.result(timeout=timeout)
         except futures.TimeoutError as e:
             coordination.raise_with_cause(coordination.OperationTimedOut,
-                                          utils.exception_message(e),
+                                          encodeutils.exception_to_unicode(e),
                                           cause=e)
 
     def done(self):
