@@ -60,6 +60,8 @@ class TestAPI(testscenarios.TestWithScenarios,
                    'bad_url': 'mysql://localhost:1'}),
         ('zookeeper', {'url': os.getenv("TOOZ_TEST_ZOOKEEPER_URL"),
                        'bad_url': 'zookeeper://localhost:1'}),
+        ('etcd', {'url': os.getenv("TOOZ_TEST_ETCD_URL"),
+                  'bad_url': 'etcd://localhost:1'})
     ]
 
     def assertRaisesAny(self, exc_classes, callable_obj, *args, **kwargs):
